@@ -10,26 +10,27 @@ WWW::Grooveshark - Perl wrapper for the Grooveshark API
 
 =head1 VERSION
 
-This document describes C<WWW::Grooveshark> version 0.01 (July 7, 2009).
+This document describes C<WWW::Grooveshark> version 0.02 (July 22, 2009).
 
 The latest version is hosted on Google Code as part of
-L<http://elementsofpuzzle.googlecode.com/>.
+L<http://elementsofpuzzle.googlecode.com/>.  Significant changes are also
+contributed to CPAN: http://search.cpan.org/dist/WWW-Grooveshark/.
 
 =cut
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 $VERSION = eval $VERSION;
 
 =head1 SYNOPSIS
 
-Basic use is demonstrated here.  See L</"API METHODS"> for details.
+Basic use is demonstrated here.  See L</API METHODS> for details.
 
   use WWW::Grooveshark;
 
   my $gs = WWW::Grooveshark->new(https => 1, agent => "my-nice-robot/0.1");
 
-  my $r = $gs->session_start(apiKey => $secret);
-  die $r->fault_line if $r->is_fault;
+  my $r;
+  $r = $gs->session_start(apiKey => $secret) or die $r->fault_line;
   
   for($gs->search_songs(query => "The Beatles", limit => 10)->songs) {
       printf("%s", $_->{songName});
@@ -38,7 +39,7 @@ Basic use is demonstrated here.  See L</"API METHODS"> for details.
       printf(" <%s>\n", $_->{liteUrl});
   }
   
-  $gs->session_destroy;
+  # session automatically ended by destructor
 
 =head1 DESCRIPTION
 
@@ -59,6 +60,7 @@ you to the official API page, which seems to still be in beta.
 use Carp;
 use Digest::MD5 qw(md5_hex);
 use JSON::Any;
+use URI::Escape;
 
 use WWW::Grooveshark::Response qw(:fault);
 
@@ -74,15 +76,21 @@ possible through key-value options.
 
 =item WWW::Grooveshark->new( %OPTIONS )
 
-Prepares a new L<WWW::Grooveshark> object with the specified options, which are
+Prepares a new C<WWW::Grooveshark> object with the specified options, which are
 passed in as key-value pairs, as in a hash.  Accepted options are:
 
 =over 4
 
+=item I<https>
+
+Whether or not to use HTTPS for API calls.  Defaults to false, i.e. just use
+HTTP.
+
 =item I<service>
 
 The hostname to use for the Grooveshark API service.  Defaults to
-"api.grooveshark.com".
+"api.grooveshark.com" unless C<staging> is true, in which case it defaults to
+"staging.api.grooveshark.com".
 
 =item I<path>
 
@@ -92,10 +100,11 @@ Path (relative to the hostname) to request for API calls.  Defaults to "ws".
 
 Version of the Grooveshark API you plan on using.  Defaults to 1.0.
 
-=item I<https>
+=item I<query_string>
 
-Whether or not to use HTTPS for API calls.  Defaults to false, i.e. just use
-HTTP.
+The query string to include in API call requests.  May be blank.  Defaults to
+"json" so that the full default API root URL becomes
+E<lt>http://api.grooveshark.com/ws/1.0/?jsonE<gt>.
 
 =item I<agent>
 
@@ -137,19 +146,41 @@ sub new {
 	croak $@ if $@;
 	my $ua = $ua_class->new(%$ua_args);
 
+	my $default_service = $opts{staging} ?
+	                      'staging.api.grooveshark.com' :
+		                  'api.grooveshark.com';
+
 	return bless({
-		_ua          => $ua,
-		_service     => $opts{service}     || 'api.grooveshark.com',
-		_path        => $opts{path}        || 'ws',
-		_api_version => $opts{api_version} || '1.0',
-		_https       => $opts{https}       || '0',
-		_session_id  => undef,
-		_json        => new JSON::Any,
+		_ua           => $ua,
+		_service      => $opts{service}      || $default_service,
+		_path         => $opts{path}         || 'ws',
+		_api_version  => $opts{api_version}  || '1.0',
+		_query_string => $opts{query_string} || 'json',
+		_https        => $opts{https}        || 0,
+		_session_id   => undef,
+		_json         => new JSON::Any,
 	}, $pkg);
 }
 
 =back
 
+=head1 DESTRUCTOR
+
+I like code that cleans up after itself.  If a program starts by creating a
+session, it's only logical that it should finish by ending it.  But should it
+be up to the programmer to manage when that happens?  Anyone can be forgetful.
+
+Enter the destructor.  Continue explicitly cleaning up, but if you forget to
+do so, the destructor has your back.  When a C<WWW::Grooveshark> object gets
+garbage collected, it will destroy its session if any.
+
+=cut
+
+sub DESTROY {
+	my $self = shift;
+	$self->session_destroy if $self->sessionID;	
+}
+
 =head1 MANAGEMENT METHODS
 
 The following methods do not issue any API calls but deal with management of
@@ -176,17 +207,18 @@ sub sessionID {
 
 The methods listed here directly wrap the methods of Groveshark's JSON-RPC
 API.  As you may have noticed, there is a very complex mapping between the
-API's official methods and those of this interface: simply replace the
-period ('.') with an underscore ('_').  As with the constructor, pass
-arguments as hash-like key-value pairs, so for example, to get the 11th through
-20th most popular songs, I would:
+API's official methods and those of this interface: replace the period ('.')
+with an underscore ('_').  As with the constructor, pass arguments as
+hash-like key-value pairs, so for example, to get the 11th through 20th most
+popular songs, I would:
 
   my $response = $gs->popular_getSongs(limit => 10, page => 2);
 
 All API methods return L<WWW::Grooveshark::Response> objects, even in case of
-errors.  Make a habit of checking that method calls were successful:
+errors, but their boolean evaluation is L<overload>ed to give false for fault
+responses.  Make a habit of checking that method calls were successful:
 
-  die $response->fault_line if $response->is_fault;
+  die $response->fault_line unless $response;
 
 Access result elements by using the key as the method name.  In list context,
 dereferencing takes place automagically, saving you a few characters:
@@ -194,7 +226,7 @@ dereferencing takes place automagically, saving you a few characters:
   my @songs = $response->songs;
 
 But after this first "layer" you're stuck dealing with hashrefs, as in the
-L</"SYNOPSIS"> (though perhaps this will change in the future if I'm up to it):
+L</SYNOPSIS> (though perhaps this will change in the future if I'm up to it):
 
   for(@songs) {
       printf("%s", $_->{songName});
@@ -271,7 +303,7 @@ sub artist_about {
 
 =item $gs->artist_getAlbums( artistID => $ARTIST_ID [, limit => $LIMIT ] [, page => $PAGE ] )
 
-Returns the albums of the artist with the specified $RTIST_ID, as well as
+Returns the albums of the artist with the specified $ARTIST_ID, as well as
 album meta-information.
 
 =cut
@@ -445,8 +477,7 @@ sub playlist_create {
 =item $gs->playlist_delete( playlistID => $PLAYLIST_ID )
 
 Deletes the playlist with the specified $PLAYLIST_ID.  Requires being
-authenticated as the playlist's creator.  (But at the time of this writing,
-this didn't seem to work as expected due to a possible server-side bug.)
+authenticated as the playlist's creator.
 
 =cut
 
@@ -501,8 +532,7 @@ sub playlist_removeSong {
 =item $gs->playlist_rename( playlistID => $PLAYLIST_ID , name => $NAME )
 
 Renames the playlist with the specified $PLAYLIST_ID to $NAME.  Requires being
-authenticated as the playlist's creator.  (But at the time of this writing,
-this didn't seem to work as expected due to a possible server-side bug.)
+authenticated as the playlist's creator.
 
 =cut
 
@@ -633,11 +663,37 @@ sub search_songs {
 
 =over 4
 
+=item $gs->service_getMethods( )
+
+Gets a list of the methods supported by the service, as well as the names of
+their parameters.  Calling this method doesn't require a session.
+
+=cut
+
+sub service_getMethods {
+	my($self, %args) = @_;
+	my $ret = $self->_call('service.getMethods', %args);
+	return $ret;
+}
+
+=item $gs->service_getVersion( )
+
+Gets the version of the API supported by the service.  Calling this method
+doesn't require a session.
+
+=cut
+
+sub service_getVersion {
+	my($self, %args) = @_;
+	my $ret = $self->_call('service.getVersion', %args);
+	return $ret;
+}
+
 =item $gs->service_ping( )
 
-Checks that the service is alive.  Seems to be the only method that doesn't
-require a session.  Useful for testing (and for getting a "Hello, world"
-greeting in some language).
+Checks that the service is alive.  Calling this method doesn't require a
+session.  Useful for testing (and for getting a "Hello, world" greeting in
+some language).
 
 =cut
 
@@ -700,7 +756,7 @@ sub session_destroy {
 	my $ret = $self->_call('session.destroy', %args);
 	
 	# kill the stored session ID if destroying was successful
-	$self->{_session_id} = undef unless $ret->is_fault;
+	$self->{_session_id} = undef if $ret;
 		
 	return $ret;
 }
@@ -734,7 +790,7 @@ sub session_get {
 	my $ret = $self->_call('session.get', %args);
 	
 	# save the session ID given in the response
-	$self->{_session_id} = $ret->sessionID unless $ret->is_fault;
+	$self->{_session_id} = $ret->sessionID if $ret;
 	
 	return $ret;
 }
@@ -795,14 +851,14 @@ sub session_start {
 	
 	my $ret = $self->_call('session.start', %args);
 	
-	if($ret->is_fault) {
-		# restore old session ID
-		$self->{_session_id} = $old_session_id;
-	}
-	else {
+	if($ret) {
 		# save the session ID given in the response
 		$self->{_session_id} = $ret->sessionID;
 	}
+	else {
+		# restore old session ID
+		$self->{_session_id} = $old_session_id;
+	}
 	
 	return $ret;
 }
@@ -922,7 +978,7 @@ applications.  This method is experimental: use it at your own risk.
 sub song_getWidgetEmbedCodeFbml {
 	my $ret = shift->song_getWidgetEmbedCode(@_);
 
-	unless($ret->is_fault) {
+	if($ret) {
 		my $code = $ret->{result}->{embed};
 		$code =~ /<embed (.*?)>\s*<\/embed>/;		
 		$code = "<fb:swf swf$1 />";
@@ -947,10 +1003,56 @@ sub song_unfavorite {
 
 =back
 
+=head2 TINYSONG
+
+=over 4
+
+=item $gs->tinysong_create( songID => $SONG_ID | ( query => $QUERY [, useFirstResult => $USE_FIRST_RESULT ] ) )
+
+Creates a tiny URL that links to the song with the specified $SONG_ID.  The
+method seems to also allow searching (if a $QUERY and whether to
+$USE_FIRST_RESULT are specified), but this form appears to be buggy at the
+time of this writing, and is discouraged.
+
+=cut
+
+sub tinysong_create {
+	my($self, %args) = @_;
+	my $ret = $self->_call('tinysong.create', %args);
+	return $ret;
+}
+
+=item $gs->tinysong_getExpandedUrl( tinySongUrl => $TINYSONG_URL )
+
+Expands a TinySong URL into the full URL to which it redirects.
+
+=cut
+
+sub tinysong_getExpandedUrl {
+	my($self, %args) = @_;
+	my $ret = $self->_call('tinysong.getExpandedUrl', %args);
+	return $ret;
+}
+
+=back
+
 =head2 USER
 
 =over 4
 
+=item $gs->user_about( $user_id => $USER_ID )
+
+Returns information about the user with the specified $USER_ID, such as
+username, date joined, etc.
+
+=cut
+
+sub user_about {
+	my($self, %args) = @_;
+	my $ret = $self->_call('user.about', %args);
+	return $ret;
+}
+
 =item $gs->user_getFavoriteSongs( $user_id => $USER_ID [, limit => $LIMIT ] [, page => $PAGE ] )
 
 Returns songs (and meta-information) from the favorite list of the user with
@@ -985,15 +1087,23 @@ sub user_getPlaylists {
 sub _call {
 	my($self, $method, %param) = @_;
 
+#	print STDERR "Called $method\n";
+
 	my $req = {
 		header     => {sessionID => $self->sessionID},
 		method     => $method,
 		parameters => \%param,
 	};
 
+#	use Data::Dumper; print STDERR Dumper($req);
+
 	my $json = $self->{_json}->encode($req);
-	my $url = sprintf("%s://%s/%s/%s/", ($self->{_https} ? 'https' : 'http'),
+	my $url = sprintf('%s://%s/%s/%s/', ($self->{_https} ? 'https' : 'http'),
 		map($self->{$_}, qw(_service _path _api_version)));
+	if(my $q = $self->{_query_string}) {
+		$q = uri_escape($q);
+		$url .= '?' . $q;
+	}
 	my $response = $self->{_ua}->post($url,
 		'Content-Type' => 'text/json',
 		'Content'      => $json,
@@ -1014,6 +1124,8 @@ sub _call {
     	};
 	}
 
+#	use Data::Dumper; print STDERR Dumper($ret);
+
 	return WWW::Grooveshark::Response->new($ret);
 }
 
@@ -1027,8 +1139,11 @@ L<http://www.grooveshark.com/>, L<WWW::Grooveshark::Response>, L<WWW::TinySong>
 
 =head1 BUGS
 
-Please report them!  Create an issue at
-L<http://elementsofpuzzle.googlecode.com/> or drop me an e-mail.
+Please report them!  The preferred way to submit a bug report for this module
+is through CPAN's bug tracker:
+http://rt.cpan.org/Public/Dist/Display.html?Name=WWW-Grooveshark.  You may
+also create an issue at http://elementsofpuzzle.googlecode.com/ or drop me an
+e-mail.
 
 =head1 AUTHOR
 
